#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import logging
from logging import handlers
import os


LOGGER = logging.getLogger("redylitics")
LOGGER.setLevel(logging.INFO)
log_file_name = os.environ.get("REDYLITICS_LOG_FILE_NAME", "redylitics.log")
formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
handler = handlers.RotatingFileHandler(log_file_name, maxBytes=5000000, backupCount=5)
handler.setFormatter(formatter)
LOGGER.addHandler(handler)


GIF = base64.b64decode("R0lGODlhAQABAIAAAP///////yH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
import re
import os

import redis


REDYLITICS_PAGEVIEW_CLIENT_WRITE_CHANNEL_REGEX = os.environ.get("REDYLITICS_PAGEVIEW_CLIENT_WRITE_CHANNEL_REGEX", ".*")
REDYLITICS_PAGEVIEW_CLIENT_WRITE_DATE_REGEX = os.environ.get("REDYLITICS_PAGEVIEW_CLIENT_WRITE_DATE_REGEX", ".*")
REDYLITICS_PAGEVIEW_CLIENT_READ_CHANNEL_REGEX = os.environ.get("REDYLITICS_PAGEVIEW_CLIENT_READ_CHANNEL_REGEX", ".*")
REDYLITICS_PAGEVIEW_CLIENT_READ_DATE_REGEX = os.environ.get("REDYLITICS_PAGEVIEW_CLIENT_READ_DATE_REGEX", ".*")
REDYLITICS_PAGEVIEW_CLIENT_DATE_EXTRACTION_REGEX = os.environ.get("REDYLITICS_PAGEVIEW_CLIENT_DATE_EXTRACTION_REGEX", "\d{8}")

VIDEO_EVENT_REGEX = os.environ.get("REDYLITICS_VIDEO_EVENT_REGEX", ".*")

VIDEO_CLIENT_PREFIX = os.environ.get("REDYLITICS_VIDEO_CLIENT_PREFIX", "video")

VIDEO_AD_CLIENT_PREFIX = os.environ.get("REDYLITICS_VIDEO_AD_CLIENT_PREFIX", "video_ad")

REDIS_HOST = os.environ.get("REDYLITICS_REDIS_HOST", "localhost")
REDIS_PORT = os.environ.get("REDYLITICS_REDIS_PORT", 6379)
REDIS_DB = os.environ.get("REDYLITICS_REDIS_DB", 0)

REDIS_KWARGS = {
    "host": REDIS_HOST,
    "port": REDIS_PORT,
    "db": REDIS_DB,
}


class BaseRedisClient(object):
    """a thin wrapper around the `redis.StrictRedis` client
    """

    def __init__(self, **kwargs):
        """initializes the client object with a `.connection` attribute

        :param kwargs: keyword arguments for building a `redis.StrictRedis` client
        :type kwargs: dict
        """
        if len(kwargs):
            redis_kwargs = kwargs
        else:
            redis_kwargs = REDIS_KWARGS
        self.connection = redis.StrictRedis(**redis_kwargs)


class PageviewClient(BaseRedisClient):
    """extends the base `BaseRedisClient` object for working with pageview data
    """

    WRITE_CHANNEL_REGEX = re.compile(REDYLITICS_PAGEVIEW_CLIENT_WRITE_CHANNEL_REGEX)

    WRITE_DATE_REGEX = re.compile(REDYLITICS_PAGEVIEW_CLIENT_WRITE_DATE_REGEX)

    READ_CHANNEL_REGEX = re.compile(REDYLITICS_PAGEVIEW_CLIENT_READ_CHANNEL_REGEX)

    READ_DATE_REGEX = re.compile(REDYLITICS_PAGEVIEW_CLIENT_READ_DATE_REGEX)

    DATE_EXTRACTION_REGEX = re.compile(REDYLITICS_PAGEVIEW_CLIENT_DATE_EXTRACTION_REGEX)

    def update_channel(self, channel, increment_by):
        """increments a channel value if the channel name, date and increment value are valid

        :param channel: a channel name + date string
        :type channel: str

        :param increment_by: the value to increment by
        :type increment_by: int

        :return: was the operation was successful?
        :rtype: bool
        """
        channel_is_valid = self.WRITE_CHANNEL_REGEX.match(channel) is not None
        if not channel_is_valid:
            raise Exception("The channel name supplied is not valid: it must be a lowercase site name")

        increment_by_is_valid = isinstance(increment_by, int)
        if not increment_by_is_valid:
            raise Exception("The increment value is not valid: it must be an integer")

        try:
            date_string = re.findall(self.WRITE_DATE_REGEX, channel)[0]
        except (TypeError, AttributeError, IndexError):
            raise Exception("The date string supplied is not valid: it must be a valid `YYYYmmdd` string")

        date_is_valid = self.validate_date(date_string)
        if not date_is_valid:
            raise Exception("The date string supplied is not valid: it must be a valid `YYYYmmdd` string")

        return self.connection.incrby(channel, increment_by)

    def read_channel(self, channel):
        """gets the value for a given channel on either a specific date or wildcard date if both are valid

        :param channel: a channel name + date/wildcard string
        :type channel: str

        :return: the set of values requested
        :rtype: dict
        """
        channel_is_valid = self.READ_CHANNEL_REGEX.match(channel) is not None
        if not channel_is_valid:
            raise Exception("The channel name supplied is not valid: it must be a lowercase site name")

        try:
            date_string = re.findall(self.READ_DATE_REGEX, channel)[0]
        except (TypeError, AttributeError, IndexError):
            raise Exception("The date string supplied is not valid: it must be a valid `YYYYmmdd` or `YYYYmm*` string")

        date_is_valid = self.validate_date(date_string)
        if not date_is_valid:
            raise Exception("The date string supplied is not valid: it must be a valid `YYYYmmdd` or `YYYYmm*` string")

        keys = self.connection.keys(channel)
        values = self.connection.mget(keys)
        for index, value in enumerate(values):
            values[index] = int(value)
        dates = []
        for key in keys:
            matches = self.DATE_EXTRACTION_REGEX.findall(key)
            if len(matches):
                date = datetime.datetime.strptime(matches[0], "%Y%m%d").strftime("%Y-%m-%d")
                dates.append(date)
        return dict(zip(dates, values))

    @staticmethod
    def validate_date(date_string):
        """validates a date string as being either YYYYmmdd or YYYYmm*

        :param date_string: the date string to be validated
        :type date_string: str

        :return: is the date valid?
        :rtype: bool
        """
        try:
            _ = datetime.datetime.strptime(date_string, '%Y%m%d')
            return True
        except (TypeError, ValueError):
            try:
                _ = datetime.datetime.strptime(date_string, '%Y%m*')
                return True
            except (TypeError, ValueError):
                return False


class BaseVideoClient(BaseRedisClient):
    """extends the base `BaseRedisClient` object for working with video type data
    """

    PREFIX = ""

    EVENT_REGEX = re.compile(VIDEO_EVENT_REGEX)

    def update_video(self, video, event, increment_by):
        """updates a video event hash value if the video, event and value are valid

        :param video: the id of the video
        :type video: int

        :param event: the name of the event
        :type event: str

        :param increment_by: the value to increment by
        :type increment_by: int

        :return: was the operation was successful?
        :rtype: bool
        """
        video_is_valid = isinstance(video, int)
        if not video_is_valid:
            raise Exception("The video identifier supplied is not valid: it must be an integer")

        event_is_valid = self.EVENT_REGEX.match(event) is not None
        if not event_is_valid:
            raise Exception(
                "The event name supplied is not valid: it must be 'start', 'complete', 'firstQuartile', "
                "'midpoint', or 'thirdQuartile'")

        increment_by_is_valid = isinstance(increment_by, int)
        if not increment_by_is_valid:
            raise Exception("The increment value is not valid: it must be an integer")

        keyed_video = self.update_video_id(video)

        return self.connection.hincrby(keyed_video, event, increment_by)

    def read_video(self, video, event=None):
        """gets the value of a video and optionally limited to a hash key

        :param video: the id of the video
        :type video: int

        :param event: the name of the event (optional)
        :type event: str

        :return: a hash of event values
        :rtype: dict
        """
        video_is_valid = isinstance(video, int)
        if not video_is_valid:
            raise Exception("The video identifier supplied is not valid: it must be an integer")

        event_is_valid = event is None or self.EVENT_REGEX.match(event) is not None
        if not event_is_valid:
            raise Exception(
                "The event name supplied is not valid: it must be 'start', 'complete', 'firstQuartile', "
                "'midpoint', or 'thirdQuartile'")

        keyed_video = self.update_video_id(video)

        video_hash = self.connection.hgetall(keyed_video)
        for key, value in video_hash.items():
            video_hash[key] = int(value)
        if event:
            return {event: video_hash.get(event)}
        return video_hash

    def update_video_id(self, video):
        """adds the client type prefix the to the video id

        :param video: the id of the video
        :type video: int

        :return: the updated video id
        :rtype: str
        """
        return "{}.{}".format(self.PREFIX, video)


class VideoClient(BaseVideoClient):
    """
    a client specifically for videos
    """

    PREFIX = VIDEO_CLIENT_PREFIX


class VideoAdClient(BaseVideoClient):
    """
    a client specifically for video ads
    """

    PREFIX = VIDEO_AD_CLIENT_PREFIX



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from collections import Counter
import json
import os
import re

import gevent
from gevent import queue

from redylitics.endpoints import update_channels, update_videos, update_video_ads, read_channels, read_videos, read_video_ads
from redylitics.utils import LOGGER, GIF

try:
    from urllib.parse import parse_qs
except ImportError:
    from urlparse import parse_qs


# init the gevent queue
CHANNEL_QUEUE = queue.Queue()
VIDEO_QUEUE = queue.Queue()
VIDEO_AD_QUEUE = queue.Queue()
FLUSH_INTERVAL = os.environ.get("REDYLITICS_FLUSH_INTERVAL", 10)  # seconds

# endpoint regexes
REDYLITICS_REFERER_EXTRACT_REGEX = os.environ.get("REDYLITICS_REFERER_EXTRACT_REGEX", ".*")
REFERER_EXTRACT_REGEX = re.compile(REDYLITICS_REFERER_EXTRACT_REGEX)

UPDATE_CHANNEL_REGEX = re.compile(r"^/pageview\.gif$")
UPDATE_VIDEO_REGEX = re.compile(r"^/video\.gif$")
UPDATE_VIDEO_AD_REGEX = re.compile(r"^/video-ad\.gif$")

READ_CHANNEL_REGEX = re.compile(r"^/pageview\.json$")
READ_VIDEO_REGEX = re.compile(r"^/video\.json$")
READ_VIDEO_AD_REGEX = re.compile(r"^/video-ad\.json$")


# update page views
def channel_counts():
    while 1:
        gevent.sleep(FLUSH_INTERVAL)
        channels = Counter()
        while 1:
            try:
                channel = CHANNEL_QUEUE.get_nowait()
                channels[channel] += 1
            except queue.Empty:
                break
            except Exception as e:
                LOGGER.exception(e)
                break
        if len(channels):
            gevent.spawn(update_channels, channels)


# update videos
def video_counts():
    while 1:
        gevent.sleep(FLUSH_INTERVAL)
        videos = Counter()
        while 1:
            try:
                video = VIDEO_QUEUE.get_nowait()
                videos[video] += 1
            except queue.Empty:
                break
            except Exception as e:
                LOGGER.exception(e)
                break
        if len(videos):
            gevent.spawn(update_videos, videos)


# update video ads
def video_ad_counts():
    while 1:
        gevent.sleep(FLUSH_INTERVAL)
        video_ads = Counter()
        while 1:
            try:
                ad = VIDEO_AD_QUEUE.get_nowait()
                video_ads[ad] += 1
            except queue.Empty:
                break
            except Exception as e:
                LOGGER.exception(e)
                break
        if len(video_ads):
            gevent.spawn(update_video_ads, video_ads)


# create the wait loops
gevent.spawn(channel_counts)
gevent.spawn(video_counts)
gevent.spawn(video_ad_counts)


# main loop
def application(env, start_response):
    # get the request path
    path = env["PATH_INFO"].lower()

    # get the query params
    try:
        params = parse_qs(env["QUERY_STRING"])
    except KeyError:
        params = None

    # get the http referer header
    try:
        referer = env["HTTP_REFERER"]
        referer = REFERER_EXTRACT_REGEX.findall(referer)[0]
    except (KeyError, IndexError):
        referer = None

    # update page views
    if UPDATE_CHANNEL_REGEX.match(path) and referer:
        start_response("200 OK", [("Content-Type", "image/gif")])
        yield GIF
        CHANNEL_QUEUE.put(referer)

    # update videos
    elif UPDATE_VIDEO_REGEX.match(path) and referer and params:
        start_response("200 OK", [("Content-Type", "image/gif")])
        yield GIF
        video = params.get("video", [None])[0]
        event = params.get("event", [None])[0]
        VIDEO_QUEUE.put((video, event))

    # update video ads
    elif UPDATE_VIDEO_AD_REGEX.match(path) and referer and params:
        start_response("200 OK", [("Content-Type", "image/gif")])
        yield GIF
        video = params.get("video_ad", [None])[0]
        event = params.get("event", [None])[0]
        VIDEO_AD_QUEUE.put((video, event))

    # read page views
    elif READ_CHANNEL_REGEX.match(path) and params:
        channel = params.get("channel", [None])[0]
        res, ok = read_channels(channel)
        if ok:
            start_response("200 OK", [("Content-Type", "application/json")])
            yield json.dumps(res)
        else:
            start_response("400 Bad Request", [("Content-Type", "application/json")])
            json.dumps({"error": res})

    # read videos
    elif READ_VIDEO_REGEX.match(path) and params:
        video = params.get("video", [None])[0]
        event = params.get("event", [None])[0]
        res, ok = read_videos(video, event)
        if ok:
            start_response("200 OK", [("Content-Type", "application/json")])
            yield json.dumps(res)
        else:
            start_response("400 Bad Request", [("Content-Type", "application/json")])
            json.dumps({"error": res})

    # read video ads
    elif READ_VIDEO_AD_REGEX.match(path) and params:
        video = params.get("video_ad", [None])[0]
        event = params.get("event", [None])[0]
        res, ok = read_video_ads(video, event)
        if ok:
            start_response("200 OK", [("Content-Type", "application/json")])
            yield json.dumps(res)
        else:
            start_response("400 Bad Request", [("Content-Type", "application/json")])
            json.dumps({"error": res})

    # bologna
    else:
        start_response("403 Forbidden", [("Content-Type", "text/plain")])
        yield "(\/) (°,,,°) (\/)"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import date, datetime

from redylitics.clients import PageviewClient, VideoClient, VideoAdClient
from redylitics.utils import LOGGER


pageview_client = PageviewClient()
video_client = VideoClient()
video_ad_client = VideoAdClient()


def update_channels(counter):
    """attempts to update a channel's pageview

    :param counter: the pageviews counter from the wsgi process
    :type counter: collections.Counter
    """
    t = date.today()
    today = datetime(t.year, t.month, t.day).strftime("%Y%m%d")
    for channel, incr in counter.items():
        channel = "{}{}".format(channel, today)
        try:
            yield pageview_client.update_channel(channel, incr)
        except Exception as e:
            LOGGER.exception(e)


def update_videos(counter):
    """attempts to update a video's event counts

    :param counter: the video counter from the wsgi process
    :type counter: collections.Counter
    """
    for (video, event), incr in counter.items():
        try:
            yield video_client.update_video(video, event, incr)
        except Exception as e:
            LOGGER.exception(e)


def update_video_ads(counter):
    """attempts to update a video ad's event counts

    :param counter: the video ad counter from the wsgi process
    :type counter: collections.Counter
    """
    for (video, event), incr in counter.items():
        try:
            yield video_ad_client.update_video(video, event, incr)
        except Exception as e:
            LOGGER.exception(e)


def read_channels(channel):
    """gets the dates and pageviews for a given channel + date string

    :param channel: the name of the channel + a date string
    :type channel: str

    :return: date: value key pairs and an is ok? value
    :rtype: dict, bool
    """
    try:
        res = pageview_client.read_channel(channel)
        return res, True
    except Exception as e:
        return str(e), False


def read_videos(video, event=None):
    """gets the counts for events for a given video

    :param video: the id of the video
    :type video: int

    :param event: the name of the event
    :type event: str

    :return: event: value key pairs and an is ok? value
    :rtype: dict, bool
    """
    try:
        video = int(video)
        res = video_client.read_video(video, event)
        return res, True
    except Exception as e:
        return str(e), False


def read_video_ads(ad, event=None):
    """gets the counts for events for a given video ad

    :param ad: the id of the video ad
    :type ad: int

    :param event: the name of the event
    :type event: str

    :return: event: value key pairs and an is ok? value
    :rtype: dict, bool
    """
    try:
        ad = int(ad)
        res = video_ad_client.read_video(ad, event)
        return res, True
    except Exception as e:
        return str(e), False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from collections import Counter
from datetime import date, datetime
from unittest import TestCase

from .clients import PageviewClient, VideoClient, VideoAdClient
from .endpoints import update_channels, update_videos, update_video_ads, read_channels, read_videos, read_video_ads


class PageviewClientTests(TestCase):
    def setUp(self):
        self.client = PageviewClient()
        print(self.client.WRITE_CHANNEL_REGEX.pattern)
        print(self.client.WRITE_DATE_REGEX.pattern)
        print(self.client.READ_CHANNEL_REGEX.pattern)
        print(self.client.READ_DATE_REGEX.pattern)
        print(self.client.DATE_EXTRACTION_REGEX.pattern)

    def tearDown(self):
        self.client.connection.flushdb()
    
    def test_update_channel(self):
        # good data
        self.assertTrue(self.client.update_channel("theonion20150101", 100))
        self.assertTrue(self.client.update_channel("clickhole20150101", 100))
        self.assertTrue(self.client.update_channel("avclub20150101", 100))
        self.assertTrue(self.client.update_channel("onionstudios20150101", 100))

        with self.assertRaises(Exception):
            # bad dates - jan 99th
            self.client.update_channel("theonion20150199", 100)
            self.client.update_channel("clickhole20150199", 100)
            self.client.update_channel("avclub20150199", 100)
            self.client.update_channel("onionstudios20150199", 100)

            # incomplete dates
            self.client.update_channel("theonion201501", 100)
            self.client.update_channel("clickhole201501", 100)
            self.client.update_channel("avclub201501", 100)
            self.client.update_channel("onionstudios201501", 100)

            # bad values - strings
            self.client.update_channel("theonion20150101", "100")
            self.client.update_channel("clickhole20150101", "100")
            self.client.update_channel("avclub20150101", "100")
            self.client.update_channel("onionstudios20150101", "100")

            # bad channel name
            self.client.update_channel("barforama20150101", 100)

    def test_read_channel(self):
        _ = self.client.update_channel("theonion20170101", 100)
        _ = self.client.update_channel("theonion20170102", 100)

        # good channel keys
        data = self.client.read_channel("theonion20170101")
        self.assertEqual(data, dict([("2017-01-01", 100)]))
        data = self.client.read_channel("theonion20170102")
        self.assertEqual(data, dict([("2017-01-02", 100)]))
        data = self.client.read_channel("theonion201701*")
        self.assertAlmostEqual(data, dict([("2017-01-02", 100), ("2017-01-01", 100)]))

        with self.assertRaises(Exception):
            # bad date
            _ = self.client.read_channel("theonion20170199")

            # incomplete date
            _ = self.client.read_channel("theonion2017*")

            # bad channel name
            _ = self.client.read_channel("barforama20170101")


class VideoClientTests(TestCase):
    def setUp(self):
        self.client = VideoClient()

    def tearDown(self):
        self.client.connection.flushdb()

    def test_update_video(self):
        # good data
        self.assertTrue(self.client.update_video(123, "start", 10))
        self.assertTrue(self.client.update_video(123, "complete", 10))
        self.assertTrue(self.client.update_video(123, "firstQuartile", 10))
        self.assertTrue(self.client.update_video(123, "midpoint", 10))
        self.assertTrue(self.client.update_video(123, "thirdQuartile", 10))

        with self.assertRaises(Exception):
            # bad video id
            self.client.update_video("123", "start", 10)
            self.client.update_video("this-is-a-video", "start", 10)

            # bad event
            self.client.update_video(123, "hooray!", 1)

            # bad value
            self.client.update_video(123, "start", 1.3)

    def test_read_video(self):
        _ = self.client.update_video(312, "start", 10)
        _ = self.client.update_video(312, "complete", 10)
        _ = self.client.update_video(312, "firstQuartile", 10)
        _ = self.client.update_video(312, "midpoint", 10)
        _ = self.client.update_video(312, "thirdQuartile", 10)

        # good keys
        data = self.client.read_video(312)
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})
        data = self.client.read_video(312, "start")
        self.assertEqual(data, {"start": 10})

        with self.assertRaises(Exception):
            _ = self.client.read_video("123", "start")
            _ = self.client.read_video("this-is-a-video")
            _ = self.client.read_video(312, "barf")


class VideoAdClientTests(VideoClientTests):
    def setUp(self):
        self.client = VideoAdClient()


class ReadChannelsEndpointTests(TestCase):
    def setUp(self):
        self.client = PageviewClient()

    def tearDown(self):
        self.client.connection.flushdb()

    def test_read_channels(self):
        key = "theonion20170101"
        _ = self.client.update_channel(key, 100)

        data = self.client.read_channel(key)
        self.assertEqual(data, {"2017-01-01": 100})

        res, ok = read_channels(key)
        self.assertTrue(ok)
        self.assertEqual(res, {"2017-01-01": 100})


class ReadVideoEndpointTests(TestCase):
    def setUp(self):
        self.client = VideoClient()

    def tearDown(self):
        self.client.connection.flushdb()

    def test_read_videos(self):
        _ = self.client.update_video(312, "start", 10)
        _ = self.client.update_video(312, "complete", 10)
        _ = self.client.update_video(312, "firstQuartile", 10)
        _ = self.client.update_video(312, "midpoint", 10)
        _ = self.client.update_video(312, "thirdQuartile", 10)

        data = self.client.read_video(312)
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})

        res, ok = read_videos(312)
        self.assertTrue(ok)
        self.assertEqual(res, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})


class ReadVideoAdEndpointTests(TestCase):
    def setUp(self):
        self.client = VideoAdClient()

    def tearDown(self):
        self.client.connection.flushdb()

    def test_read_video_ads(self):
        _ = self.client.update_video(312, "start", 10)
        _ = self.client.update_video(312, "complete", 10)
        _ = self.client.update_video(312, "firstQuartile", 10)
        _ = self.client.update_video(312, "midpoint", 10)
        _ = self.client.update_video(312, "thirdQuartile", 10)

        data = self.client.read_video(312)
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})

        res, ok = read_video_ads(312)
        self.assertTrue(ok)
        self.assertEqual(res, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})


class UpdateChannelsTests(TestCase):
    def setUp(self):
        self.client = PageviewClient()

    def tearDown(self):
        self.client.connection.flushdb()

    def test_update_channels(self):
        page_views = Counter({"theonion": 100})
        results = update_channels(page_views)
        for res in results:
            self.assertTrue(res)

        t = date.today()
        today = datetime(t.year, t.month, t.day).strftime("%Y%m%d")
        key = "theonion{}".format(today)
        data = self.client.read_channel(key)

        key = datetime(t.year, t.month, t.day).strftime("%Y-%m-%d")
        self.assertEqual(data, {key: 100})


class UpdateVideosTests(TestCase):
    def setUp(self):
        self.client = VideoClient()

    def tearDown(self):
        self.client.connection.flushdb()

    def test_update_videos(self):
        videos = Counter({
            (312, "start"): 10,
            (312, "complete"): 10,
            (312, "firstQuartile"): 10,
            (312, "midpoint"): 10,
            (312, "thirdQuartile"): 10,
        })
        results = update_videos(videos)
        for res in results:
            self.assertTrue(res)

        data = self.client.read_video(312)
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})


class UpdateVideoAdsTests(TestCase):
    def setUp(self):
        self.client = VideoAdClient()

    def tearDown(self):
        self.client.connection.flushdb()

    def test_update_videos(self):
        ads = Counter({
            (312, "start"): 10,
            (312, "complete"): 10,
            (312, "firstQuartile"): 10,
            (312, "midpoint"): 10,
            (312, "thirdQuartile"): 10,
        })
        results = update_video_ads(ads)
        for res in results:
            self.assertTrue(res)

        data = self.client.read_video(312)
        self.assertEqual(data, {"firstQuartile": 10, "start": 10, "midpoint": 10, "complete": 10, "thirdQuartile": 10})



from distutils.core import setup


setup(
    name="redylitics",
    version="0.0.9",
    description="Read and Write Event Data to Redis",
    author="Vince Forgione",
    author_email="vforgione@theonion.com",
    packages=["redylitics"],
    install_requires=[
        "gevent==1.0.1",
        "redis==2.10.3",
    ],
    url="https://github.com/theonion/redylitics"
)
